@@ -2,13 +2,20 @@ package DateTime::Duration;
 
 use strict;
 use warnings;
+use namespace::autoclean;
 
-our $VERSION = '1.25';
+our $VERSION = '1.39';
 
 use Carp ();
 use DateTime;
 use DateTime::Helpers;
-use Params::Validate qw( validate SCALAR );
+use DateTime::Types;
+use Params::ValidationCompiler qw( validation_for );
+
+BEGIN {
+    my $has = eval { require Sub::Util; 1 };
+    sub HAS_SUB_UTIL () {$has}
+}
 
 use overload (
     fallback => 1,
@@ -19,58 +26,74 @@ use overload (
     'cmp'    => '_compare_overload',
 );
 
-use constant MAX_NANOSECONDS => 1_000_000_000;    # 1E9 = almost 32 bits
+sub MAX_NANOSECONDS () {1_000_000_000}    # 1E9 = almost 32 bits
 
 my @all_units = qw( months days minutes seconds nanoseconds );
 
-# XXX - need to reject non-integers but accept infinity, NaN, &
-# 1.56e+18
-sub new {
-    my $class = shift;
-    my %p     = validate(
-        @_, {
-            years        => { type => SCALAR, default => 0 },
-            months       => { type => SCALAR, default => 0 },
-            weeks        => { type => SCALAR, default => 0 },
-            days         => { type => SCALAR, default => 0 },
-            hours        => { type => SCALAR, default => 0 },
-            minutes      => { type => SCALAR, default => 0 },
-            seconds      => { type => SCALAR, default => 0 },
-            nanoseconds  => { type => SCALAR, default => 0 },
+{
+    my %units = map {
+        $_ => {
+
+            # XXX - what we really want is to accept an integer, Inf, -Inf,
+            # and NaN, but I can't figure out how to accept NaN since it never
+            # compares to anything.
+            type    => t('Defined'),
+            default => 0,
+            }
+        } qw(
+        years
+        months
+        weeks
+        days
+        hours
+        minutes
+        seconds
+        nanoseconds
+    );
+
+    my $check = validation_for(
+        ( HAS_SUB_UTIL ? ( name => '_check_new_params' ) : () ),
+        params => {
+            %units,
             end_of_month => {
-                type  => SCALAR, default => undef,
-                regex => qr/^(?:wrap|limit|preserve)$/
+                type     => t('EndOfMonthMode'),
+                optional => 1,
             },
+        },
+    );
+
+    sub new {
+        my $class = shift;
+        my %p     = $check->(@_);
+
+        my $self = bless {}, $class;
+
+        $self->{months} = ( $p{years} * 12 ) + $p{months};
+
+        $self->{days} = ( $p{weeks} * 7 ) + $p{days};
+
+        $self->{minutes} = ( $p{hours} * 60 ) + $p{minutes};
+
+        $self->{seconds} = $p{seconds};
+
+        if ( $p{nanoseconds} ) {
+            $self->{nanoseconds} = $p{nanoseconds};
+            $self->_normalize_nanoseconds;
         }
-    );
+        else {
 
-    my $self = bless {}, $class;
+            # shortcut - if they don't need nanoseconds
+            $self->{nanoseconds} = 0;
+        }
 
-    $self->{months} = ( $p{years} * 12 ) + $p{months};
+        $self->{end_of_month} = (
+              defined $p{end_of_month} ? $p{end_of_month}
+            : $self->{months} < 0      ? 'preserve'
+            :                            'wrap'
+        );
 
-    $self->{days} = ( $p{weeks} * 7 ) + $p{days};
-
-    $self->{minutes} = ( $p{hours} * 60 ) + $p{minutes};
-
-    $self->{seconds} = $p{seconds};
-
-    if ( $p{nanoseconds} ) {
-        $self->{nanoseconds} = $p{nanoseconds};
-        $self->_normalize_nanoseconds;
+        return $self;
     }
-    else {
-
-        # shortcut - if they don't need nanoseconds
-        $self->{nanoseconds} = 0;
-    }
-
-    $self->{end_of_month} = (
-          defined $p{end_of_month} ? $p{end_of_month}
-        : $self->{months} < 0      ? 'preserve'
-        :                            'wrap'
-    );
-
-    return $self;
 }
 
 # make the signs of seconds, nanos the same; 0 < abs(nanos) < MAX_NANOS
@@ -255,7 +278,7 @@ sub multiply {
 }
 
 sub compare {
-    my ( $class, $dur1, $dur2, $dt ) = @_;
+    my ( undef, $dur1, $dur2, $dt ) = @_;
 
     $dt ||= DateTime->now;
 
@@ -313,13 +336,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 DateTime::Duration - Duration objects for date math
 
 =head1 VERSION
 
-version 1.25
+version 1.39
 
 =head1 SYNOPSIS
 
@@ -586,17 +611,25 @@ Comparison is B<not> overloaded. If you attempt to compare durations
 using C<< <=> >> or C<cmp>, then an exception will be thrown!  Use the
 C<compare()> class method instead.
 
-=head1 SUPPORT
-
-Support for this module is provided via the datetime@perl.org email
-list. See http://lists.perl.org/ for more details.
-
 =head1 SEE ALSO
 
 datetime@perl.org mailing list
 
 http://datetime.perl.org/
 
+=head1 SUPPORT
+
+Support for this module is provided via the datetime@perl.org email
+list. See http://lists.perl.org/ for more details.
+
+Bugs may be submitted through L<the RT bug tracker|http://rt.cpan.org/Public/Dist/Display.html?Name=DateTime>
+(or L<bug-datetime@rt.cpan.org|mailto:bug-datetime@rt.cpan.org>).
+
+There is a mailing list available for users of this distribution,
+L<mailto:datetime@perl.org>.
+
+I am also usually active on IRC as 'drolsky' on C<irc://irc.perl.org>.
+
 =head1 AUTHOR
 
 Dave Rolsky <autarch@urth.org>
